<?php
namespace Findologic\Search\Controller\Export;

use Findologic\Search\Helper\Xml as Xml;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\ObjectManager;
use Magento\Store\Model\ScopeInterface;
use Magento\Catalog\Model\Product\Attribute\Source\Status;
use Magento\Catalog\Model\Product\Visibility;
use Magento\Catalog\Model\ResourceModel\Product\Collection;

class ExportController extends Action
{
    /**
     * @var string
     */
    private $shopKey;

    /**
     * @var int
     */
    private $storeId;

    /**
     * @var int
     */
    private $count;

    /**
     * @var int
     */
    private $start;

    /**
     * @var int
     */
    private $total;

    /**
     * @var Collection
     */
    private $productsCollection;

    /**
     * @param Context $context
     * @param Collection $productsCollection
     */
    public function __construct(Context $context, Collection $productsCollection)
    {
        parent::__construct($context);

        $this->productsCollection = $productsCollection;
    }

    /**
     * Exports data using shop key.
     */
    public function execute()
    {
        $request = $this->getRequest();
        $this->shopKey = $request->getParam('shopkey', false);
        $this->start = $request->getParam('start', false);
        $this->count = $request->getParam('count', false);
        $this->storeId = $this->getStoreId($this->shopKey);

        $this->validateInput();

        $products = $this->getAllValidProducts();
        $xml = $this->createXml($products);

        if ($request->getParam('validate', false)) {
            $this->validateXml($xml);
        }

        header('Content-Type: application/xml; charset=utf-8');
        echo $xml;
        die;
    }

    /**
     * Validates whether all input parameters are supplied.
     */
    private function validateInput()
    {
        $message = '';

        if (!$this->shopKey) {
            $message = 'Parameter "shopkey" is missing! ';
        }

        if (!$this->storeId) {
            $message .= 'Parameter "shopkey" is not configured for any store! ';
        }

        if ($this->start === false || $this->start < 0) {
            $message .= 'Parameter "start" is missing or less than 0! ';
        }

        if (!$this->count || $this->count < 01) {
            $message .= 'Parameter "count" is missing or less than 1!';
        }

        if ($message) {
            die($message);
        }
    }

    private function validateXml($xml)
    {
        libxml_use_internal_errors(true);
        /** @var \DOMDocument $dom */
        $dom = new DOMDocument();
        $dom->loadXML($xml);
        $path = 'https://raw.githubusercontent.com/FINDOLOGIC/xml-export/master/src/main/resources/findologic.xsd';
        if (!$dom->schemaValidate($path)) {
            $errors = libxml_get_errors();
            foreach ($errors as $error) {
                $return = "<br/>\n";
                switch ($error->level) {
                    case LIBXML_ERR_WARNING:
                        $return .= "<b>Warning $error->code</b>: ";
                        break;
                    case LIBXML_ERR_ERROR:
                        $return .= "<b>Error $error->code</b>: ";
                        break;
                    case LIBXML_ERR_FATAL:
                        $return .= "<b>Fatal Error $error->code</b>: ";
                        break;
                }

                $return .= trim($error->message);
                if ($error->file) {
                    $return .= " in <b>$error->file</b>";
                }

                echo $return . " on line <b>$error->line</b>\n";
            }

            die;
        }
    }

    /**
     * Get store id for specified shop key.
     *
     * @param string $shopKey
     * @return boolean|integer store id if found; otherwise, false.
     */
    private function getStoreId($shopKey)
    {
        //TODO Get rid of Object Manager.
        $om = ObjectManager::getInstance();
        $storeManager = $om->get('\Magento\Store\Model\StoreManagerInterface');
        
        $stores = $storeManager->getStores();
        
        $scopeConfig = $om->get('\Magento\Framework\App\Config\ScopeConfigInterface');
        
        foreach ($stores as $store) {
            $storeId = $store->getId();

            $keyValue = $scopeConfig->getValue('findologic/findologic_group/shopkey',
                ScopeInterface::SCOPE_STORE, $storeId);

            if ($keyValue === $shopKey) {
                return $storeId;
            }
        }

        return false;
    }

    /**
     * Get all products that passed validation for current selected store.
     *
     * @return Collection $productsCollection
     */
    private function getAllValidProducts()
    {
        $this->productsCollection
            ->setStoreId($this->storeId)
            ->addStoreFilter($this->storeId)
            ->addAttributeToSelect('*')
            ->addAttributeToFilter('status', Status::STATUS_ENABLED)
            ->addAttributeToFilter('visibility',
                [
                'in' =>
                    [
                        Visibility::VISIBILITY_IN_SEARCH,
                        Visibility::VISIBILITY_IN_CATALOG,
                        Visibility::VISIBILITY_BOTH,
                    ]
                ]
            )
            ->joinField(
                'qty',
                'cataloginventory_stock_item',
                'qty',
                'product_id = entity_id',
                'is_in_stock = 1',
                'left'
            )
            ->getSelect()
            ->limit($this->count, $this->start);

        $this->total = $this->productsCollection->getSize();

        return $this->productsCollection->load();
    }

    /**
     * Create XML file.
     *
     * @param Collection $products
     * @return string
     */
    private function createXml($products)
    {
        $xml = new Xml();
        return $xml->buildXmlDoc(
            array(
                'storeId' => $this->storeId,
                'shopkey' => $this->shopKey,
                'count' => $this->count,
                'start' => $this->start,
                'total' => $this->total,
            ), $products
        );
    }
}
