<?php
namespace Findologic\Search\Helper;

use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Request\Http;
use Magento\Framework\App\ObjectManager;
use Magento\Store\Model\ScopeInterface;
use Magento\Catalog\Model\Session;
use Magento\Customer\Model\Session as CustomerSession;
use Magento\Framework\App\Helper\Context;

class TrackingScripts extends AbstractHelper
{
    /**
     * @var array
     */
    private $data;

    /**
     * @var Session
     */
    private $session;

    /**
     * Used for caching categories retrieved from database.
     *
     * @var array
     */
    private static $categories = array();

    public function  __construct(Context $context, Session $session)
    {
        parent::__construct($context);

        $this->session = $session;
    }

    /**
     * Renders needed scripts for current page.
     *
     * @param Http $request
     * @return string
     */
    public function renderScripts(Http $request)
    {
        $findologic = $request->get('findologic');
        $result = '';

        if ($findologic != 'off' && $this->getShopKey()) {

            // Inserting js files for every page
            $result .= $this->headJs();
        }

        return $result;
    }

    /**
     * Get configuration data to start tracking
     */
    private function getConfData()
    {
        /** @var $session CustomerSession*/
        $om = ObjectManager::getInstance();
        $session = $om->get('\Magento\Customer\Model\Session');
        $customer = $session->getCustomer();

        $userGroup = $om->get('\Magento\Customer\Model\Group')
            ->load((int)$customer->getGroupId())
            ->getCustomerGroupCode();
        $shopKey = $this->getShopKey();
        $userGroupHash = $this->userGroupToHash($shopKey, $userGroup);
        
        $this->data['USERGROUP_HASH'] = $session->isLoggedIn() ? $userGroupHash : '';
        $this->data['HASHED_SHOPKEY'] = strtoupper(md5($shopKey));
    }

    /**
     * Returns tracking js script for head (all pages).
     *
     * @return string
     */
    private function headJs()
    {
        $this->getConfData();

        $headJs = sprintf(
            '<script type="text/javascript">
            (function() {
                var mainUrl = "https://cdn.findologic.com/static/%s/main.js?usergrouphash=%s";
                var loader  = document.createElement("script"); 
                loader.type = "text/javascript"; 
                loader.async = true;
                loader.src = "https://cdn.findologic.com/static/loader.min.js";
                var s = document.getElementsByTagName("script")[0];
                loader.setAttribute("data-fl-main", mainUrl);
                s.parentNode.insertBefore(loader, s);
            })();
            </script>', $this->data['HASHED_SHOPKEY'], $this->data['USERGROUP_HASH']);

        return $headJs;
    }

    /**
     * @return string Reads plugin configuration and returns shop key for current shop.
     */
    private function getShopKey()
    {
        $om = ObjectManager::getInstance();
        $scopeConfig = $om->get('\Magento\Framework\App\Config\ScopeConfigInterface');
        $storeManager = $om->get('\Magento\Store\Model\StoreManagerInterface');
        $storeId = $storeManager->getStore()->getId();

        return $scopeConfig->getValue('findologic/findologic_group/shopkey',
                            ScopeInterface::SCOPE_STORE, $storeId);
    }

    /**
     * Gets category path as string separated by provided separator.
     *
     * @param string $categoryPathIds Identifiers separated by slash. Example: 154/6584/653/8
     * @param string $separator Separator for generated path between category names
     * @param bool $wrap Indicates whether to wrap each category name with quotes
     * @param bool $addSlashes
     * @return string
     */
    public function getCategoryPath($categoryPathIds, $separator = ', ', $wrap = true, $addSlashes = true)
    {
        $holder = array();
        $categoryIds = explode('/', $categoryPathIds);
        $n = 0;
        foreach ($categoryIds as $catId) {
            // Skip 'root' and 'default' category and add up to 5 categories.
            if ($n < 5 && $catId != 1 && $catId != 2) {
                $category = $this->getCategory($catId);
                $categoryName = $addSlashes ? addslashes($category->getName()) : $category->getName();
                $holder[] = $wrap ? '"' . $categoryName . '"' : $categoryName;

                $n++;
            }
        }

        return implode($separator, $holder);
    }

    /**
     * Gets category from local cache.
     *
     * @param int $id Category identifier
     * @return mixed
     */
    public function getCategory($id)
    {
        if (!array_key_exists($id, self::$categories)) {
            $il = ObjectManager::getInstance();
            self::$categories[$id] = $il->create('Magento\Catalog\Model\ResourceModel\Category\Collection')
                            ->addAttributeToSelect(array('is_active', 'name'))
                            ->addFieldToFilter('entity_id', $id)
                            ->getFirstItem();
        }

        return self::$categories[$id];
    }

    /**
     * Returns hash key merged from shop key and user group.
     *
     * @param string $shopKey
     * @param string $userGroup
     * @return string
     */
    public function userGroupToHash($shopKey, $userGroup)
    {
        return base64_encode($shopKey ^ $userGroup);
    }
}
