<?php

namespace Findologic\Search\Model;

use Magento\Catalog\Model\Product;
use Magento\ConfigurableProduct\Model\Product\Type\Configurable;
use Magento\Bundle\Model\Product\Type;
use Magento\Framework\App\ObjectManager;
use Findologic\Search\Model\DTO\PriceDto;

class Price {

    /**
     * @param Product $product
     */
    public function __construct(Product $product)
    {
        $this->product = $product;
    }

    /**
     * Returns actual product price, special product price, and base product price
     * for given product instance.
     *
     * @return PriceDto
     */
    public function getProductPrice()
    {
        $productTypeId = $this->product->getTypeId();

        switch ($productTypeId) {
            case Configurable::TYPE_CODE:
                $price = $this->getConfigurableProductPrice();
                break;
            case Type::TYPE_CODE:
                $price = $this->getBundleProductPrice();
                break;
            default:
                $price = $this->getSimpleProductPrice();
        }

        return $price;
    }

    /**
     * Returns actual product price, special product price and base product price
     * for bundle product type.
     *
     * @return PriceDto
     */
    private function getBundleProductPrice()
    {
        $productFinalPrice = $this->product->getPriceInfo()->getPrice('final_price');

        $priceDto = new PriceDto();

        $priceDto->actualPrice = $productFinalPrice->getMinimalPrice()->getValue();
        $priceDto->specialPrice = $priceDto->actualPrice;
        $priceDto->basePrice = $productFinalPrice->getMinimalPrice()->getValue();

        return $priceDto;
    }

    /**
     * Returns actual product price, special product price, and base product price
     * for configurable product type.
     *
     * @return PriceDto
     */
    private function getConfigurableProductPrice()
    {
        $priceDto = new PriceDto();

        $priceDto->actualPrice = $this->determineActualProductPrice(
            [
                $this->getProductVariantPrices(),
                $this->product->getFinalPrice(),
                $this->product->getSpecialPrice()
            ]);
        $priceDto->specialPrice = $priceDto->actualPrice;
        $priceDto->basePrice = $this->product->getFinalPrice();

        return $priceDto;
    }

    /**
     * Returns actual product price, special product price, and base product price
     * for simple product type.
     *
     * @return PriceDto
     */
    private function getSimpleProductPrice()
    {
        $finalSpecialPrice = $this->getFinalSpecialPrice();
        $productPrice = $this->product->getPrice();

        $priceDto = new PriceDto();

        $priceDto->actualPrice = $this->determineActualProductPrice([$finalSpecialPrice, $productPrice]);
        $priceDto->specialPrice = ($finalSpecialPrice) ? $this->getFinalSpecialPrice() : null;
        $priceDto->basePrice = $this->product->getPrice();

        return $priceDto;
    }

    /**
     * Return an array with prices for products which belong to given configurable product.
     *
     * @return array
     */
    private function getProductVariantPrices()
    {
        $productVariantPrices = [];
        $variants = $this->product->getTypeInstance(true)->getUsedProducts($this->product);

        foreach ($variants as $variant) {
            $productVariantPrices[] = floatval($variant->getPrice());
        }

        return $productVariantPrices;
    }

    /**
     * Actual product price is equal to the lowest price of all possible product prices.
     * Possible product prices could be: configured price, special price, catalog rule price and some special prices
     * for specific types of products such as: product variant price for configurable products.
     *
     * @param array $prices
     * @return string
     */
    private function determineActualProductPrice($prices)
    {
        $possiblePrices = [];

        foreach ($prices as $price) {
            if ($price != null) {
                array_push($possiblePrices, $price);
            }
        }

        return (count($possiblePrices)) ? min($possiblePrices) : 0;
    }

    /**
     * Return product price after applying catalog rules to it.
     *
     * @return string|null
     */
    private function getProductPriceAfterPriceRule()
    {
        //TODO Try to avoid this ObjectManager, and move it to constructor.
        $om = ObjectManager::getInstance();
        $priceRule = $om->get('Magento\CatalogRule\Model\Rule');

        $catalogRulePrice = $priceRule->calcProductPriceRule
        (
            $this->product,
            $this->product->getPrice()
        );

        return $catalogRulePrice;
    }


    /**
     * Return special product price if it is eligible.
     * Special product price is eligible if current date is between special price from and to date.
     *
     * @return string|null
     */
    private function getSpecialPrice()
    {
        $specialPriceFromDate = strtotime($this->product->getSpecialFromDate());
        $specialPriceToDate = strtotime($this->product->getSpecialToDate());
        $currentDate = strtotime(date('Y-m-d h:m:s'));

        if ($specialPriceToDate) {
            return ($currentDate <= $specialPriceToDate && $currentDate >= $specialPriceFromDate)
                ? $this->product->getSpecialPrice()
                : null;
        } else {
            return ($currentDate >= $specialPriceFromDate)
                ? $this->product->getSpecialPrice()
                : null;
        }
    }

    /**
     * Return final special price of the product.
     * If both special price and catalog rule price exists special price is the lower price of these two price.
     *
     * @return string|null
     */
    private function getFinalSpecialPrice()
    {
        $specialPrice = $this->getSpecialPrice();
        $catalogRulePrice = $this->getProductPriceAfterPriceRule();

        $finalPrice = null;

        if ($specialPrice == null && $catalogRulePrice != null) {
           $finalPrice = $catalogRulePrice;
        }

        if ($specialPrice != null && $catalogRulePrice == null) {
            $finalPrice = $specialPrice;
        }

        if ($specialPrice != null && $catalogRulePrice != null) {

            if ($specialPrice < $catalogRulePrice) {
                $finalPrice = $specialPrice;
            } else {
                $finalPrice = $catalogRulePrice;
            }
        }

        return $finalPrice;
    }
}